# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

MIN_ENSEMBL_RELEASE = 40
MAX_ENSEMBL_RELEASE = 115
MAX_PLANTS_ENSEMBL_RELEASE = 58


def check_release_number(release):
    """
    Check to make sure a release is in the valid range of
    Ensembl releases.
    """
    try:
        release = int(release)
    except (ValueError, TypeError):
        raise ValueError("Invalid Ensembl release: %s" % release)

    if release < MIN_ENSEMBL_RELEASE:
        raise ValueError(
            "Invalid Ensembl releases %d, must be greater than %d"
            % (release, MIN_ENSEMBL_RELEASE)
        )
    return release
